package com.qmth.cqb.utils.word;

import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.ZipUtils;
import com.qmth.cqb.utils.enums.QuesUnit;

import freemarker.template.Configuration;
import freemarker.template.Template;
import net.sf.saxon.TransformerFactoryImpl;
import net.sourceforge.jeuclid.LayoutContext;
import net.sourceforge.jeuclid.context.LayoutContextImpl;
import net.sourceforge.jeuclid.context.StyleAttributeLayoutContext;
import net.sourceforge.jeuclid.converter.Converter;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.docx4j.Docx4J;
import org.docx4j.TraversalUtil;
import org.docx4j.XmlUtils;
import org.docx4j.convert.in.FlatOpcXmlImporter;
import org.docx4j.convert.in.xhtml.XHTMLImporterImpl;
import org.docx4j.convert.out.HTMLSettings;
import org.docx4j.convert.out.common.preprocess.PartialDeepCopy;
import org.docx4j.dml.CTBlip;
import org.docx4j.dml.wordprocessingDrawing.Inline;
import org.docx4j.finders.ClassFinder;
import org.docx4j.jaxb.Context;
import org.docx4j.math.CTOMath;
import org.docx4j.math.CTOMathPara;
import org.docx4j.openpackaging.exceptions.Docx4JException;
import org.docx4j.openpackaging.io3.stores.ZipPartStore;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.openpackaging.parts.Part;
import org.docx4j.openpackaging.parts.WordprocessingML.BinaryPartAbstractImage;
import org.docx4j.openpackaging.parts.WordprocessingML.MainDocumentPart;
import org.docx4j.openpackaging.parts.relationships.Namespaces;
import org.docx4j.openpackaging.parts.relationships.RelationshipsPart;
import org.docx4j.relationships.Relationship;
import org.docx4j.wml.*;
import org.dom4j.Namespace;
import org.dom4j.io.SAXReader;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.InputSource;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipOutputStream;

/**
 * docx处理工具类 Created by songyue on 17/3/10.
 */
public final class DocxProcessUtil {

	private static final Logger logger = LoggerFactory.getLogger(DocxProcessUtil.class);
	
    public static final LayoutContext IMG_LAYOUT = new StyleAttributeLayoutContext(
            LayoutContextImpl.getDefaultLayoutContext(), "2em", java.awt.Color.BLACK);
    
    public static Configuration configuration;
    
    public static final String IMG_OUT_TYPE = "image/png";

    public static final String ENCODING = "utf-8";

    public static final String BASE64_HEADER = "data:image/png;base64,";

    public static final String TEMP_FILE_IMP = "docxImport/";

    public static final String TEMP_FILE_EXP = "docxExport/";

    public static final String OMML2MML_XSL = "OMML2MML.XSL";

    public static final String PAPER_TEMPLATE = "paper_template.ftl";

    public static final String ANSWER_TEMPLATE = "answer_template.ftl";

    public static final String DOCX_SUFFIX = ".docx";

    public static final String BODY_HEADER = "<w:body xmlns:w=\"http://schemas.openxmlformats.org/wordprocessingml/2006/main\">";

    public static final String BODY_TAIL = "</w:body>";

    public static final String REL_ID_HEADER = "rId";

    static {
        init();
    }

    /**
     * 初始化文件夹
     */
    public static void init() {
        File temp_file_exp = new File(TEMP_FILE_EXP);
        File temp_file_img = new File(TEMP_FILE_IMP);
        // 如果输出目标文件夹不存在，则创建
        if (!temp_file_img.exists()) {
            temp_file_img.mkdirs();
        }
        if (!temp_file_exp.exists()) {
            temp_file_exp.mkdirs();
        }
    }

    /**
     * 获取段落的所有文本
     *
     * @param p
     * @return
     */
    public static String getPText(P p) {
        String returnText = "";
        List<Object> tList = getAllElementFromObject(p, Text.class);
        for (Object obj : tList) {
            Text text = (Text) obj;
            returnText += text.getValue().trim();
        }
        return returnText.trim();
    }

    /**
     * 校验段落中是否含有公式或图片
     *
     * @param p
     * @return
     */
    public static boolean isText(P p) {
        List<Object> mathList = getAllElementFromObject(p, CTOMath.class);
        List<Object> mathParaList = getAllElementFromObject(p, CTOMathPara.class);
        List<Object> drawList = getAllElementFromObject(p, Drawing.class);
        if (mathParaList.size() > 0 || mathList.size() > 0 || drawList.size() > 0) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * 获取文档包副本(去除段落)
     *
     * @param wordMLPackage
     * @return
     * @throws Exception
     */
    public static WordprocessingMLPackage getTmpPackage(WordprocessingMLPackage wordMLPackage) throws Exception {
        // 添加关系样式信息
        Set<String> relationshipTypes = new HashSet<String>();
        relationshipTypes.add(wordMLPackage.getMainDocumentPart().getRelationshipType());
        // 深拷贝
        WordprocessingMLPackage tmpWordMlPackage = (WordprocessingMLPackage) PartialDeepCopy
                .process(wordMLPackage, relationshipTypes);
        // 移除所有段落
        initTmpPackage(tmpWordMlPackage);

        return tmpWordMlPackage;
    }

    /**
     * 初始化文档包(去除所有段落)
     *
     * @param wordMLPackage
     * @return
     * @throws Exception
     */
    public static void initTmpPackage(WordprocessingMLPackage wordMLPackage) throws Exception {
        // 移除所有段落
        List<Object> contentList = wordMLPackage.getMainDocumentPart().getContent();
        contentList.removeAll(contentList);
    }

    /**
     * 获得段落wordml
     *
     * @param p
     * @return
     */
    public static String getPWordMl(P p) throws Exception {
        return XmlUtils.marshaltoString(p);
    }

    /**
     * 转换wordMl为html
     *
     * @param wordMl
     * @return
     * @throws Exception
     */
    public static String docx2Html(String wordMl, WordprocessingMLPackage wordMLPackage) throws Exception {

        initTmpPackage(wordMLPackage);

        // 转换wordml为完整的package对象
        MainDocumentPart mdp = wordMLPackage.getMainDocumentPart();
        Object p = XmlUtils.unmarshalString(wordMl);

        mdp.getContent().add(p);

        // 转换公式为图片
        replaceMath2Image(wordMLPackage, (P) p);

        // 设置导出临时html参数
        HTMLSettings htmlSettings = Docx4J.createHTMLSettings();
        htmlSettings.setImageDirPath(TEMP_FILE_IMP + "_files");
        htmlSettings.setImageTargetUri(TEMP_FILE_IMP.substring(TEMP_FILE_IMP.lastIndexOf("/") + 1) + "_files");
        htmlSettings.setWmlPackage(wordMLPackage);
        String tmpHtmlPath = TEMP_FILE_IMP + UUID.randomUUID().toString() + ".html";
        File tmpHtml = new File(tmpHtmlPath);
        OutputStream os = new java.io.FileOutputStream(tmpHtml);
        Docx4J.toHTML(htmlSettings, os, Docx4J.FLAG_EXPORT_PREFER_XSL);
        os.flush();

        // 格式化html代码
        String htmlStr = formatHtmlByPath(tmpHtmlPath);

        // 删除临时html文件
        FileUtils.deleteQuietly(tmpHtml);

        return htmlStr;
    }

    /**
     * 替换所有公式为图片
     *
     * @param wordMLPackage
     * @throws Exception
     */
    public static void replaceAllMath2Image(WordprocessingMLPackage wordMLPackage) throws Exception {
        List<Object> pList = getAllElementFromObject(wordMLPackage.getMainDocumentPart(), P.class);
        for (Object pObject : pList) {
            P p = (P) pObject;
            List<Object> pContent = p.getContent();
            int index = 0;
            for (Object child : pContent) {
                if (child instanceof ContentAccessor) {
                    index++;
                    continue;
                }
                if (child instanceof JAXBElement)
                    child = ((JAXBElement<?>) child).getValue();
                if (child.getClass().equals(CTOMath.class)) {
                    String omml = XmlUtils.marshaltoString(child, true, true, Context.jc,
                            "http://schemas.openxmlformats.org/officeDocument/2006/math", "oMath", CTOMath.class);
                    byte[] imgByte = convertMathml2Img(omml2mml(omml));
                    pContent.set(index, newImage(wordMLPackage, imgByte, "math" + (index), "math" + (index)));
                }
                index++;
            }
        }
    }

    /**
     * 替换公式为图片
     *
     * @param p
     * @throws Exception
     */
    public static void replaceMath2Image(WordprocessingMLPackage wordMLPackage, P p) throws Exception {
        List<Object> pContent = p.getContent();
        int index = 0;
        for (Object child : pContent) {
            if (child instanceof ContentAccessor) {
                index++;
                continue;
            }
            if (child instanceof JAXBElement)
                child = ((JAXBElement<?>) child).getValue();
            if (child.getClass().equals(CTOMath.class)) {
                String omml = XmlUtils.marshaltoString(child, true, true, Context.jc,
                        "http://schemas.openxmlformats.org/officeDocument/2006/math", "oMath", CTOMath.class);
                byte[] imgByte = convertMathml2Img(omml2mml(omml));
                pContent.set(index, newImage(wordMLPackage, imgByte, "math" + (index), "math" + (index)));
            }else if(child.getClass().equals(CTOMathPara.class)){
                String omml = XmlUtils.marshaltoString(child, true, true, Context.jc,
                        "http://schemas.openxmlformats.org/officeDocument/2006/math", "oMathPara", CTOMathPara.class);
                byte[] imgByte = convertMathml2Img(omml2mml(omml));
                pContent.set(index, newImage(wordMLPackage, imgByte, "math" + (index), "math" + (index)));
            }
            index++;
        }
    }

    /**
     * 创建新图片
     *
     * @param wordMLPackage
     * @param bytes
     * @param filenameHint
     * @param altText
     * @return
     * @throws Exception
     */
    public static R newImage(WordprocessingMLPackage wordMLPackage, byte[] bytes, String filenameHint, String altText)
            throws Exception {
        int id1 = (int) ((Math.random() * 1000) * (Math.random() * 1000));
        int id2 = (int) ((Math.random() * 1000) * (Math.random() * 1000));

        BinaryPartAbstractImage imagePart = BinaryPartAbstractImage.createImagePart(wordMLPackage, bytes);

        Inline inline = imagePart.createImageInline(filenameHint, altText, id1, id2, false);

        ObjectFactory factory = Context.getWmlObjectFactory();
        R run = factory.createR();
        Drawing drawing = factory.createDrawing();
        run.getContent().add(drawing);
        drawing.getAnchorOrInline().add(inline);
        bytes = null;
        imagePart = null;
        return run;

    }

    /**
     * 获取wordMl固定节点下所有子节点
     *
     * @param obj
     * @param toSearch
     * @return
     */
    public static List<Object> getAllElementFromObject(Object obj, Class<?> toSearch) {

        List<Object> result = new ArrayList<Object>();
        if (obj instanceof JAXBElement)
            obj = ((JAXBElement<?>) obj).getValue();
        if (obj.getClass().equals(toSearch))
            result.add(obj);
        else if (obj instanceof ContentAccessor) {
            List<?> children = ((ContentAccessor) obj).getContent();
            for (Object child : children) {
                result.addAll(getAllElementFromObject(child, toSearch));
            }
        }
        return result;
    }

    /**
     * omml转换成mml
     *
     * @param omml
     * @return
     */
    public static String omml2mml(String omml)throws Exception {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        InputStream stylesheet = DocxProcessUtil.class.getClassLoader().getResourceAsStream(OMML2MML_XSL);
        StringReader sr = new StringReader(omml);
        InputSource ommldata = new InputSource(sr);
        StringWriter writer = new StringWriter();
        org.w3c.dom.Document document = null;
        DocumentBuilder builder = null;
        Transformer transformer = null;
        builder = factory.newDocumentBuilder();
        document = builder.parse(ommldata);
        TransformerFactory tFactory = new TransformerFactoryImpl();
        StreamSource stylesource = new StreamSource(stylesheet);
        transformer = tFactory.newTransformer(stylesource);
        DOMSource source = new DOMSource(document);
        StreamResult result = new StreamResult(writer);
        transformer.transform(source, result);
        IOUtils.closeQuietly(sr);
        IOUtils.closeQuietly(writer);
        String mml = writer.toString();
        mml = ("<mml:math" + StringUtils.substringAfter(mml, "<mml:math")).replace("mml:", "");
        return mml;
    }

    /**
     * 格式化转换后的html(html字符串)
     *
     * @param htmlStr
     * @return
     */
    public static String formatHtml(String htmlStr) throws Exception {
        org.jsoup.nodes.Document doc = Jsoup.parse(htmlStr);
        Elements divs = doc.select("div[class='document");
        doc.select("p").removeAttr("class").removeAttr("style");
        doc.select("div").removeAttr("class").removeAttr("style");
        doc.select("span").removeAttr("class").removeAttr("style");
        doc.select("span").stream().forEach(element -> {
            if (!element.hasText())
                element.remove();
        });
        if (divs.size() > 0) {
            htmlStr = divs.html();
        } else {
            htmlStr = doc.html();
        }
        return htmlStr;
    }

    /**
     * 获取html(html字符串)里面文本
     *
     * @param htmlStr
     * @return
     */
    public static String getTextInHtml(String htmlStr){
        if(!htmlStr.startsWith("<p>")){
            return htmlStr;
        }
        try {
            org.jsoup.nodes.Document doc = Jsoup.parse(htmlStr);
            String textStr = "";
            Elements links = doc.select("p").removeAttr("img");
            for (Element link : links) {
                textStr += link.text();
            }
            return textStr;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return htmlStr;
    }

    /**
     * 格式化转换后的html(html临时文件)
     *
     * @param htmlPath
     * @return
     */
    public static String formatHtmlByPath(String htmlPath) throws Exception {
        String htmlStr = "";
        File htmlFile = new File(htmlPath);
        org.jsoup.nodes.Document doc = Jsoup.parse(htmlFile, ENCODING);
        Elements divs = doc.select("div[class='document']");
        doc.select("p").removeAttr("class").removeAttr("style");
        doc.select("div").removeAttr("class").removeAttr("style");
        doc.select("span").removeAttr("class").removeAttr("style");
        doc.select("span").stream().forEach(element -> {
            if (!element.hasText() && element.childNodeSize() == 0) {
                element.remove();
            }
        });
        Elements imgs = doc.select("img");
        for (org.jsoup.nodes.Element img : imgs) {
            String imgSrc = img.attr("src");
            if (!StringUtils.isEmpty(imgSrc)) {
                String imgBase64 = new String(getBase64ByPath(htmlFile.getParent() + "/" + imgSrc));
                img.attr("src", BASE64_HEADER + imgBase64);
            }
        }
        if (divs.size() > 0) {
            htmlStr = divs.html();
        } else {
            htmlStr = doc.html();
        }
        return htmlStr;
    }

    /**
     * 转换mathml为png图片
     *
     * @param mathMlStr
     * @return
     */
    public static byte[] convertMathml2Img(String mathMlStr) throws Exception {
        File xmlFile = null;
        File imgFile = null;
        FileOutputStream xmlFos = null;
        byte[] base64Byte = new byte[0];
        xmlFile = new File(TEMP_FILE_IMP, UUID.randomUUID().toString());
        imgFile = new File(TEMP_FILE_IMP, UUID.randomUUID().toString());
        xmlFos = new FileOutputStream(xmlFile);
        IOUtils.write(mathMlStr, xmlFos, ENCODING);
        xmlFos.flush();
        Converter.getInstance().convert(xmlFile, imgFile, IMG_OUT_TYPE, IMG_LAYOUT);
        base64Byte = IOUtils.toByteArray(new FileInputStream(imgFile));
        IOUtils.closeQuietly(xmlFos);
        FileUtils.deleteQuietly(xmlFile);
        FileUtils.deleteQuietly(imgFile);
        return base64Byte;
    }

    /**
     * 通过图片路径获取base64码
     *
     * @param imgFilePath
     * @return
     */
    public static byte[] getBase64ByPath(String imgFilePath) throws Exception {
        InputStream is = null;
        byte[] base64Byte = new byte[0];
        byte[] imgByte;
        File imgFile = new File(imgFilePath);
        is = new FileInputStream(imgFile);
        imgByte = IOUtils.toByteArray(is);
        base64Byte = Base64.encodeBase64(imgByte);
        IOUtils.closeQuietly(is);
//        FileUtils.deleteQuietly(imgFile);
        return base64Byte;
    }

    /**
     * 通过图片文件获取base64码
     *
     * @param imgFile
     * @return
     */
    public static byte[] getBase64ByFile(File imgFile) throws Exception {
        InputStream is = null;
        byte[] base64Byte = new byte[0];
        byte[] imgByte;
        is = new FileInputStream(imgFile);
        imgByte = IOUtils.toByteArray(is);
        base64Byte = Base64.encodeBase64(imgByte);
        IOUtils.closeQuietly(is);
        return base64Byte;
    }

    /**
     * 格式化段落wordml
     *
     * @param pWordMl
     * @return
     * @throws Exception
     */
    public static String formatPWordMl(String pWordMl) throws Exception {
        String tmpWordMl = pWordMl.replaceAll("\\\\\"", "\"");
        SAXReader sax = new SAXReader();
        ByteArrayInputStream is = new ByteArrayInputStream(tmpWordMl.getBytes("UTF-8"));
        org.dom4j.Document document = sax.read(is);
        org.dom4j.Element root = document.getRootElement();
        List<Namespace> namespaces = root.additionalNamespaces();
        namespaces.stream().forEach(namespace -> {
            root.remove(namespace);
        });
        List<org.dom4j.Element> elements = root.elements();
        elements.stream().forEach(element ->{
        	if("w:bookmarkStart".equals(element.getQualifiedName())||
                	"w:bookmarkEnd".equals(element.getQualifiedName())){
        		root.remove(element);
        	}
        });
        return root.asXML();
    }

    /**
     * 过滤试题单元标题
     *
     * @param p
     * @param quesUnit
     * @return
     */
    public static P formatP(P p, QuesUnit quesUnit) {
        List<Object> pContent = p.getContent();
        int index = 0;
        for (Object child : pContent) {
            if (child.getClass().equals(R.class)) {

                R r = (R) child;
                List<Object> rContent = r.getContent();

                for (Object rChild : rContent) {

                    if (rChild instanceof JAXBElement)
                        rChild = ((JAXBElement<?>) rChild).getValue();

                    if (rChild.getClass().equals(Text.class)) {
                        ++index;
                        Text text = (Text) rChild;
                        String tmpText = text.getValue();
                        if (quesUnit == QuesUnit.QUES_BODY) {

                            // 过滤题干标题
                            if (tmpText.matches("^\\d{1,}\\.[\\s\\S]*")) {
                                tmpText = tmpText.replaceFirst("\\d{1,}\\.","");
                                text.setValue(tmpText);
                                return p;
                            } else if(tmpText.matches("^\\d{1,}$")){
                                tmpText = tmpText.replaceFirst("\\d{1,}","");
                                text.setValue(tmpText);
                            } else if(tmpText.contains(".")){
                                tmpText = tmpText.replaceFirst("\\.","");
                                text.setValue(tmpText);
                                return p;
                            }

                        } else if (quesUnit == QuesUnit.QUES_OPTION) {

                            // 过滤选项标题
                            if (tmpText.matches("^[A-Z]\\.[\\s\\S]*")) {
                                tmpText = tmpText.replaceFirst("[A-Z]\\.","");
                                text.setValue(tmpText);
                                return p;
                            } else if(tmpText.matches("^[A-Z]$")){
                                tmpText = tmpText.replaceFirst("[A-Z]","");
                                text.setValue(tmpText);
                            } else if(tmpText.contains(".")){
                                tmpText = tmpText.replaceFirst("\\.","");
                                text.setValue(tmpText);
                                return p;
                            }

                        } else if (quesUnit == QuesUnit.QUES_ANSWER) {

                            // 过滤答案标题
                            if(index <= 4){
                                tmpText = tmpText.replaceFirst("\\[","").replaceFirst("\\]","")
                                        .replaceFirst("答案","").replaceFirst("[:|：]","");
                                text.setValue(tmpText);
                            }else{
                                return p;
                            }
//                            if(tmpText.contains(":") || tmpText.contains("：")){
//                                if(tmpText.startsWith(":") || tmpText.startsWith("：")){
//                                    tmpText = tmpText.replaceFirst("[:|：]","");
//                                    text.setValue(tmpText);
//                                }else{
//                                    tmpText = tmpText.replaceFirst("\\[","").replaceFirst("\\]","").replaceFirst("答案","").replaceFirst("[:|：]","");
//                                    text.setValue(tmpText);
//                                }
//                                return p;
//                            } else {
//                                tmpText = tmpText.replaceFirst("\\[","").replaceFirst("\\]","").replaceFirst("答案","");
//                                text.setValue(tmpText);
//                            }
                        }
                    }
                }
            }
        }
        return p;
    }

    /**
     * 导出word
     *
     * @param dataMap
     * @param fileName
     */
    public static void exportWord(Map dataMap, String fileName,Template template)throws Exception {
        Writer out = null;
        // 输出文件
        File outFile = new File(TEMP_FILE_EXP + fileName + DOCX_SUFFIX);
        // 将模板和数据模型合并生成文件
        out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outFile), ENCODING));
        // 生成文件
        template.process(dataMap, out);
        out.flush();
        IOUtils.closeQuietly(out);
    }

    /**
     * 导出试卷
     *
     * @param dataMap
     * @param fileName
     */
    public static void exportPaper(Map dataMap, String fileName,Template template) throws Exception {
        exportWord(dataMap, fileName, template);
    }

    /**
     * 导出答案
     *
     * @param dataMap
     * @param fileName
     */
    public static void exportAnswer(Map dataMap, String fileName,Template template) throws Exception {
        exportWord(dataMap, fileName, template);
    }

    /**
     * 处理导出word中的图片
     *
     * @param fileName
     * @param wordMLPackages
     * @throws Exception
     */
    public static void processImage(String fileName, List<WordprocessingMLPackage> wordMLPackages) throws Exception {
    	logger.info(fileName+"word导出处理图片开始...");
        String filePath = TEMP_FILE_EXP + fileName + DOCX_SUFFIX;
        InputStream mainFile = new FileInputStream(filePath);
        // 以单个wordXml方式解析freemarker导出的文件
        FlatOpcXmlImporter flatOpcXmlImporter = new FlatOpcXmlImporter(mainFile);
        WordprocessingMLPackage wordMLPackage = (WordprocessingMLPackage) flatOpcXmlImporter.get();
        RelationshipsPart relationshipsPart = wordMLPackage.getMainDocumentPart().getRelationshipsPart();

        for (WordprocessingMLPackage wp : wordMLPackages) {
            // 获取资源文件存储
            ZipPartStore partStore = (ZipPartStore) wp.getSourcePartStore();
            // 获取图片资源定义
            RelationshipsPart rp = wp.getMainDocumentPart().getRelationshipsPart();
            List<Relationship> rels = rp.getRelationshipsByType(Namespaces.IMAGE);

            List<Part> parts = new ArrayList<Part>();
            for (Relationship relationship : rels) {
                parts.add(rp.getPart(relationship));
            }
            // 添加资源文件定义
            for (Part p : parts) {
                String relId = p.getSourceRelationships().get(0).getId();
                if(!relationshipsPart.isRelIdOccupied(relId)){
                    copyImage(wordMLPackage,partStore,p);
                }
            }
            // 添加资源文件存储
            //wordMLPackage.setSourcePartStore(partStore);
        }
        // 以word2007标准模式重新保存(zip包)
        OutputStream os = new java.io.FileOutputStream(filePath);
        Docx4J.save(wordMLPackage, os, Docx4J.FLAG_SAVE_ZIP_FILE);
        IOUtils.closeQuietly(os);
        logger.info(fileName+"word导出处理图片结束:"+System.currentTimeMillis());
    }

    /**
     * word合并时复制图片
     * @param wordMLPackage
     * @param partStore
     * @param p
     * @throws Exception
     */
    public static void copyImage(WordprocessingMLPackage wordMLPackage, ZipPartStore partStore, Part p)throws Exception{
        byte [] bytes = partStore.getByteArray(p.getPartName().getName().substring(1)).getBytes();
        ByteArrayInputStream bais = new ByteArrayInputStream(bytes);  
        BufferedImage bi1 = ImageIO.read(bais);
        if(bi1!=null){
        	BinaryPartAbstractImage imagePart = BinaryPartAbstractImage.createImagePart(wordMLPackage, bytes);
            imagePart.getRelLast().setId(p.getSourceRelationships().get(0).getId());
        }
        
    }
    
    /**
     * 获取word二进制数据(空文档,只有样式和资源)
     *
     * @param wordMLPackage
     * @return
     * @throws Exception
     */
    public static byte[] getPkgByte(WordprocessingMLPackage wordMLPackage) throws Exception {

        DocxProcessUtil.initTmpPackage(wordMLPackage);

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        Docx4J.save(wordMLPackage, outputStream);
        byte[] bytes = outputStream.toByteArray();

        outputStream.flush();
        IOUtils.closeQuietly(outputStream);
        return bytes;
    }

    /**
     * 通过二进制流获取word文档包
     *
     * @param bytes
     * @return
     */
    public static WordprocessingMLPackage getPkg(byte[] bytes) {
        if (bytes.length == 0)
            return null;
        InputStream inputStream = new ByteArrayInputStream(bytes);
        WordprocessingMLPackage wordMLPackage = null;
        try {
            wordMLPackage = Docx4J.load(inputStream);
        } catch (Docx4JException e) {
            e.printStackTrace();
        }
        return wordMLPackage;
    }

    /**
     * 初始化word文档图片资源路径,防止导出试卷时资源ID冲突
     *
     * @param wordMLPackage
     */
    public static void initPkgImage(WordprocessingMLPackage wordMLPackage) {
        // 获取文档中所有图片引用位置
        ClassFinder finder = new ClassFinder(CTBlip.class);
        new TraversalUtil(wordMLPackage.getMainDocumentPart().getContent(), finder);
        List<Object> blips = finder.results;
        // 获取文档中所有图片引用定义
        List<Relationship> relationships = wordMLPackage.getMainDocumentPart().getRelationshipsPart()
                .getRelationshipsByType(Namespaces.IMAGE);
        // 同步替换资源ID
        int index = 0;
        for (Relationship relationship : relationships) {
            String tmpId = relationship.getId();
            String tmp = getRldNum() + (++index);
            for (Object obj : blips) {
                if (obj.getClass().equals(CTBlip.class)) {
                    CTBlip ctBlip = (CTBlip) obj;
                    String tmpEmbed = ctBlip.getEmbed();
                    if (tmpId.equals(tmpEmbed)) {
                        relationship.setId(tmp);
                        ctBlip.setEmbed(tmp);
                    }
                }
            }
        }
    }

    /**
     * 获取RldNum
     *
     * @return
     */
    public static String getRldNum() {
        return REL_ID_HEADER + CommonUtils.getCurNum();
    }

    /**
     * 下载word文件
     * @param fileName
     * @param response
     * @throws Exception
     */
    public static void processDownload(List<String> fileNames, HttpServletResponse response) throws Exception {	
    	 String zipFileName = fileNames.get(0);
         response.setHeader("Content-Disposition","attachment; filename="+new String(zipFileName.getBytes("UTF-8"), "iso-8859-1")+".zip");
         // 设置强制下载不打开
         response.setContentType("application/octet-stream;charset=utf-8");
         ZipOutputStream out = new ZipOutputStream(response.getOutputStream());
        for(String fileName:fileNames){
            ZipUtils.doCompress(TEMP_FILE_EXP+fileName+DOCX_SUFFIX, out);
            response.flushBuffer();
        }
        out.close();
   
    }

    /**
     * html转word
     * @param html
     * @return
     * @throws Exception
     */
    public static String html2Docx(WordprocessingMLPackage wordMLPackage,String html)throws Exception {
        initTmpPackage(wordMLPackage);
        RFonts rfonts = Context.getWmlObjectFactory().createRFonts();
        rfonts.setAscii("eastAsia");
        XHTMLImporterImpl.addFontMapping("eastAsia", rfonts);
        XHTMLImporterImpl XHTMLImporter = new XHTMLImporterImpl(wordMLPackage);
        XHTMLImporter.setHyperlinkStyle("Hyperlink");
        String wordMl = "";
        wordMLPackage.getMainDocumentPart().getContent().addAll(
                    XHTMLImporter.convert( repairHtmlStr(html), TEMP_FILE_IMP) );
        // 获取word文档中所有段落
        List<Object> pList = getAllElementFromObject(wordMLPackage.getMainDocumentPart(), P.class);
        for(Object p:pList){
            wordMl += XmlUtils.marshaltoString(p);
        }
//            wordMl = formatPWordMl(wordMl);
        initPkgImage(wordMLPackage);
        return wordMl;
    }

    public static String repairHtmlStr(String htmlStr)throws Exception{
        htmlStr = htmlStr.trim();
        if(htmlStr.toLowerCase().contains("<!doctype html ")){
            int index1 = htmlStr.toLowerCase().indexOf("<!doctype html ");
            int index2 = htmlStr.indexOf('>',index1 + 1);
            htmlStr = htmlStr.substring(0, index1) + htmlStr.substring(index2 + 1);
        }
        while(htmlStr.toLowerCase().contains("<br ")){
            int index1 = htmlStr.toLowerCase().indexOf("<br ");
            int index2 = htmlStr.toLowerCase().indexOf(">",index1 + 1);
            htmlStr = htmlStr.substring(0, index1) + "<br/>" + htmlStr.substring(index2 + 1);
        }
        while(htmlStr.toLowerCase().endsWith("<br>") || htmlStr.toLowerCase().endsWith("<br/>")){
            if(htmlStr.toLowerCase().endsWith("<br>")){
                htmlStr = htmlStr.substring(0, htmlStr.length()-"<br>".length());
            }else if(htmlStr.toLowerCase().endsWith("<br/>")){
                htmlStr = htmlStr.substring(0, htmlStr.length()-"<br/>".length());
            }
        }
        htmlStr = htmlStr.replace("<br>", "<br/>").replace("<BR>", "<br/>");

        {//补全META标签
            int imgIndex = indexOfRegex(htmlStr,"<((meta)|(META)) ");
            while(imgIndex > 0){
                int flag = htmlStr.indexOf(">", imgIndex);
                if(htmlStr.charAt(flag - 1) != '/'){
                    htmlStr = htmlStr.substring(0,flag) + "/" + htmlStr.substring(flag);
                }
                imgIndex = indexOfRegex(htmlStr,"<((meta)|(META)) ",flag);
            }
        }

        {//补全img标签
            int imgIndex = indexOfRegex(htmlStr,"<((img)|(IMG)) ");
            while(imgIndex > 0){
                int flag = htmlStr.indexOf(">", imgIndex);
                if(htmlStr.charAt(flag - 1) != '/'){
                    htmlStr = htmlStr.substring(0,flag) + "/" + htmlStr.substring(flag);
                }
                imgIndex = indexOfRegex(htmlStr,"<((img)|(IMG)) ",flag);
            }
        }
        return new String(htmlStr.getBytes("UTF-8"));
    }

    /**
     * 从指定的位置开始查找第一个匹配正则表达式的字符串的位置
     * @param str
     * @param regex 正则表达式
     * @param fromIndex 指定的起始位置
     * @return
     */
    public static int indexOfRegex(String str,String regex,int fromIndex){
        int index = indexOfRegex(str.substring(fromIndex),regex);
        if(index < 0){
            return -1;
        }
        return fromIndex + index;
    }

    /**
     * 查找第一个匹配正则表达式的字符串的位置
     * @param str
     * @param regex 正则表达式
     * @return
     */
    public static int indexOfRegex(String str,String regex){
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(str);
        if(m.find()){
            return m.start();
        }else{
            return -1;
        }
    }

    public static void main(String[] args) {
        for(QuesUnit quesUnit:QuesUnit.values()){
            System.out.println(quesUnit.name()+"  "+quesUnit.getName());
        }
    }
}
