package com.qmth.cqb.paper.service;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.regex.Pattern;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.wml.P;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.gson.Gson;
import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.model.ImportPaperCheck;
import com.qmth.cqb.paper.model.ImportPaperMsg;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.question.model.QuesOption;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.question.model.QuestionType;
import com.qmth.cqb.question.service.QuesService;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.enums.PaperStatus;
import com.qmth.cqb.utils.enums.PaperType;
import com.qmth.cqb.utils.enums.QuesUnit;
import com.qmth.cqb.utils.exception.PaperException;
import com.qmth.cqb.utils.word.DocxProcessUtil;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

@Service
public class ImportPaperService {

    protected static final Logger log = LoggerFactory.getLogger(ImportPaperService.class);
    @Autowired
    PaperRepo paperRepo;

    @Autowired
    PaperDetailRepo paperDetailRepo;

    @Autowired
    PaperDetailUnitRepo paperDetailUnitRepo;

    @Autowired
    QuesService quesService;

    @Autowired
    Gson gson;

    @Autowired
    PaperDetailService paperDetailService;

    @Autowired
    PaperService paperService;

    /**
     * 获取上传文件
     *
     * @param file
     * @return
     */
    public File getUploadFile(CommonsMultipartFile file) throws Exception {
        String fileName = file.getOriginalFilename();
        File tempFile = new File(DocxProcessUtil.TEMP_FILE_IMP + fileName);
        OutputStream os = null;
        os = new FileOutputStream(tempFile);
        IOUtils.copyLarge(file.getInputStream(), os);
        IOUtils.closeQuietly(os);
        return tempFile;
    }

    /**
     * 导入试卷
     *
     * @param paperName
     * @param courseNo
     * @param file
     * @return
     */
    public Paper importPaper(Paper paper, AccessUser user,File file) throws Exception {
        paperService.checkPaperNameNew(paper.getName(), user.getRootOrgId().toString());
        return processImportPaper(paper,user,file);
    }

    /**
     * 保存导入试卷信息
     *
     * @param paper
     * @param paperDetails
     * @param paperDetailUnits
     * @param importPaperCheck
     * @throws PaperException 
     */
    public Paper savePaper(Paper paper, List<PaperDetail> paperDetails, 
    									List<PaperDetailUnit> paperDetailUnits, 
    									List<Question> questions,
    									ImportPaperCheck importPaperCheck) throws Exception {
        Paper tempPaper = null;
        if (!StringUtils.isEmpty(importPaperCheck.errorInfo))
            return null;
        if (paper != null) {
        	//总分校验
        	checkTotalScore(paper,paperDetailUnits,importPaperCheck);
        	//大题下小题数量校验
        	checkUnitNum(paperDetailUnits,importPaperCheck);
        	//校验答案是否不全
        	checkAnswerISfull(paperDetailUnits,importPaperCheck);
            tempPaper = paperRepo.save(paper);
        }
        if (paperDetails.size() > 0) {
            paperDetailRepo.save(paperDetails);
        }
        if (questions.size() > 0) {
            quesService.saveImportQuesList(questions);
        }
        if (paperDetailUnits.size() > 0) {
            paperDetailUnitRepo.save(paperDetailUnits);
        }
        paperService.formatPaper(tempPaper, null);
        return tempPaper;
    }

    /**
     * 清空试卷对象,加速回收
     * @param paper
     * @param paperDetails
     * @param paperDetailUnits
     * @param questions
     */
    public void clearPaper(Paper paper,List<PaperDetail> paperDetails,List<PaperDetailUnit> paperDetailUnits, List<Question> questions){
        paper = null;
        paperDetails.clear();
        for(PaperDetailUnit paperDetailUnit:paperDetailUnits){
            paperDetailUnit.setQuestion(null);
        }
        for(Question question:questions){
            question.setQuesPkg(null);
        }
        paperDetailUnits.clear();
        questions.clear();

    }

    /**
     * 处理导入试卷
     * 
     * @param paperName
     * @param courseNo
     * @param courseName
     * @param user
     * @param file
     * @return
     */
    public Paper processImportPaper(Paper paper,AccessUser user,File file) throws Exception {
        WordprocessingMLPackage wordMLPackage;
        WordprocessingMLPackage tmpWordMlPackage;
        WordprocessingMLPackage writePkg;
        ImportPaperCheck importPaperCheck = new ImportPaperCheck();

        wordMLPackage = WordprocessingMLPackage.load(file);
        // 初始化图片路径
        DocxProcessUtil.initPkgImage(wordMLPackage);
        // 深拷贝临时pkg与最终写入数据库pkg
        tmpWordMlPackage = DocxProcessUtil.getTmpPackage(wordMLPackage);
        writePkg = DocxProcessUtil.getTmpPackage(wordMLPackage);
        byte[] writeByte = DocxProcessUtil.getPkgByte(writePkg);
        // 获取word文档中所有段落
        List<Object> pList = DocxProcessUtil.getAllElementFromObject(wordMLPackage.getMainDocumentPart(), P.class);
        paper.setOrgId(user.getRootOrgId().toString());
        paper.setCreator(user.getName());
        // 设置试卷
        initPaper(paper, paper.getName());
        // 创建空大题类
        PaperDetail paperDetail = null;
        // 创建大题集合
        List<PaperDetail> paperDetails = new ArrayList<PaperDetail>();
        // 创建小题集合
        List<PaperDetailUnit> paperDetailUnits = new ArrayList<PaperDetailUnit>();
        // 创建试题集合
        List<Question> questions = new ArrayList<Question>();
        // 大题号
        int mainQuesNum = 0;
        // 小题号
        int subQuesNum = 0;
        for (int i = 0; i < pList.size(); i++) {
            P p = (P) pList.get(i);
            String pText = DocxProcessUtil.getPText(p);
            if (StringUtils.isEmpty(pText)) {
                // 跳过空白段落
                continue;
            } else if (isQuesHeader(pText)) {
                // 处理大题头信息
                processMainQuesHeader(pList, importPaperCheck.index, importPaperCheck);
                // 校验大题头信息
                if (!checkQuesHeader(importPaperCheck)) {
                    throw new PaperException(importPaperCheck.errorInfo);
                }
                // 创建大题类
                paperDetail = new PaperDetail();
                // 设置大题类
                initQuesHeader(paper, paperDetail, paperDetails, ++mainQuesNum, importPaperCheck);
                // 设置当前索引,防止多余循环
                i = importPaperCheck.index - 1;
            } else if (pText.matches("^\\d{1,}\\.[\\s\\S]*")
                    || (isNested(importPaperCheck) && !pText.startsWith(ImportPaperMsg.left_bracket))) {
                if(paperDetail == null){
                    throw new PaperException("导入文件格式有误,必须有大题头信息,且以 [ 开头!");
                }
                ++subQuesNum;
                // 处理试题
                // 创建小题类和试题类
                PaperDetailUnit paperDetailUnit = new PaperDetailUnit();
                Question question = new Question();
                // 初始化小题类和试题类
                initPaperDetail(paper, paperDetail, paperDetailUnit, question, subQuesNum , importPaperCheck);
                // 处理客观题
                if (importPaperCheck.quesType.equals(ImportPaperMsg.singleSelection) || importPaperCheck.quesType.equals(ImportPaperMsg.multipleSelection)) {
                    // 处理题干
                    processQuesBody(pList, importPaperCheck.index, subQuesNum, question, importPaperCheck, tmpWordMlPackage);
                    // 处理选项
                    processQuesOption(pList, importPaperCheck.index, subQuesNum, question, importPaperCheck,
                            tmpWordMlPackage);
                    // 处理尾信息
                    processQuesTail(pList, importPaperCheck.index, subQuesNum, question, paperDetailUnit,
                            importPaperCheck, tmpWordMlPackage, false);
                    // 处理选择题的option--chenken 20170425
                    processSelectOption(question);
                } else if (importPaperCheck.quesType.equals(ImportPaperMsg.nestedQuestion_word)) {
                    // 处理套题
                    processNestedQues(pList, importPaperCheck.index,question, paperDetailUnit, importPaperCheck,tmpWordMlPackage);
                } else {
                    // 处理其他题型
                    processQuesBody(pList, importPaperCheck.index, subQuesNum,question, importPaperCheck, tmpWordMlPackage);
                    processQuesTail(pList, importPaperCheck.index, subQuesNum, question, paperDetailUnit,importPaperCheck, tmpWordMlPackage, false);
                }
                // 设置WordMlPackage二进制数据
                question.setQuesPkg(writeByte);
                // 设置question与Unit集合数据
                question.setCourseNo(paper.getCourseNo());
                question.setCourseName(paper.getCourseName());
                question.setOrgId(user.getRootOrgId().toString());
                questions.add(question);
                paperDetailUnits.add(paperDetailUnit);
                // 设置当前索引,防止多余循环
                i = importPaperCheck.index - 1;
            }else if(paperDetail == null){
                throw new PaperException("导入文件格式有误,必须有大题头信息,且以 [ 开头!");
            }
            if (!StringUtils.isEmpty(importPaperCheck.errorInfo)) {
                throw new PaperException(importPaperCheck.errorInfo);
            }
        }
        if (paperDetails.size() == 0 || paperDetailUnits.size() == 0) {
            throw new PaperException("导入文件格式有误!");
        }
        paper.setPaperDetailCount(mainQuesNum);
        // 保存导入试卷信息
        paper = savePaper(paper, paperDetails, paperDetailUnits, questions, importPaperCheck);
        clearPaper(paper, paperDetails, paperDetailUnits, questions);
        wordMLPackage = null;
        tmpWordMlPackage = null;
        writePkg = null;
        writeByte = null;
        FileUtils.deleteQuietly(file);
        return paper;
    }

    public boolean isQuesHeader(String pText){
        return pText.startsWith(ImportPaperMsg.left_bracket)
                && !pText.startsWith(ImportPaperMsg.answer_word)
                && !pText.startsWith(ImportPaperMsg.score_word)
                && !pText.startsWith(ImportPaperMsg.subQuestionType_word);
    }

    /**
     * 初始化试卷信息
     *
     * @param paper
     * @param paperName
     */
    public void initPaper(Paper paper, String paperName) {
        paper.setName(paperName);
        paper.setTitle(paperName);
        paper.setPaperType(PaperType.IMPORT);
        paper.setPaperStatus(PaperStatus.DRAFT);
    }

    /**
     * 初始化大题头信息
     *
     * @param paper
     * @param paperDetail
     * @param paperDetails
     * @param mainQuesNum
     * @param importPaperCheck
     */
    public void initQuesHeader(Paper paper, PaperDetail paperDetail, List<PaperDetail> paperDetails, int mainQuesNum,
            ImportPaperCheck importPaperCheck) {

        paperDetail.setPaper(paper);
        // 设置大题信息
        paperDetail.setName(importPaperCheck.quesName);
        paperDetail.setUnitCount(Integer.parseInt(importPaperCheck.quesCount));
        paperDetail.setNumber(mainQuesNum);
        paperDetail.setCreator(paper.getCreator());
        paperDetails.add(paperDetail);

    }

    /**
     * 初始化试卷明细和试题
     *
     * @param paperDetail
     * @param paperDetailUnit
     * @param question
     * @param subQuesNum
     * @param importPaperCheck
     */
    public void initPaperDetail(Paper paper, PaperDetail paperDetail, PaperDetailUnit paperDetailUnit,
            Question question, int subQuesNum, ImportPaperCheck importPaperCheck) {

        question.setQuestionType(getQuesStructType(importPaperCheck.quesType));
        question.setScore(Double.parseDouble(importPaperCheck.quesScore));
        paperDetailUnit.setNumber(subQuesNum);
        paperDetailUnit.setQuestion(question);
        paperDetailUnit.setPaperDetail(paperDetail);
        paperDetailUnit.setQuestionType(getQuesStructType(importPaperCheck.quesType));
        paperDetailUnit.setScore(Double.parseDouble(importPaperCheck.quesScore));
        paperDetailUnit.setCreator(paper.getCreator());
        paperDetailUnit.setPaper(paper);
    }

    /**
     * 校验试题头标识
     *
     * @param importPaperCheck
     * @return
     */
    public boolean checkQuesHeader(ImportPaperCheck importPaperCheck) {

        String tmpErrorInfo = "";
        if (StringUtils.isEmpty(importPaperCheck.quesGroup)) {
            tmpErrorInfo += ImportPaperMsg.errMsg_04;

        }
        if (StringUtils.isEmpty(importPaperCheck.quesType)) {
            tmpErrorInfo += ImportPaperMsg.errMsg_05;

        }
        if (StringUtils.isEmpty(importPaperCheck.quesName)) {
            tmpErrorInfo += ImportPaperMsg.errMsg_06;

        }
        if (StringUtils.isEmpty(importPaperCheck.quesCount)) {
            tmpErrorInfo += ImportPaperMsg.errMsg_07;

        }
        if (StringUtils.isEmpty(importPaperCheck.quesScore)) {
            tmpErrorInfo += ImportPaperMsg.errMsg_08;
        }
        if (!StringUtils.isEmpty(importPaperCheck.quesType) && !CommonUtils.checkQuesType(importPaperCheck.quesType)) {
            tmpErrorInfo += ImportPaperMsg.errMsg_09;
        }
        if (StringUtils.isEmpty(tmpErrorInfo)) {
            return true;
        } else {
            importPaperCheck.setErrorInfo(tmpErrorInfo);
            return false;
        }
    }

    /**
     * 处理大题头信息
     *
     * @param pList
     * @param index
     * @param importPaperCheck
     * @throws PaperException 
     */
    public void processMainQuesHeader(List<Object> pList, int index, ImportPaperCheck importPaperCheck) throws PaperException {
        int i = 0;
        for (i = index; i < pList.size(); i++) {
            P pHeader = (P) pList.get(i);
            P pNextHeader = (P) pList.get(i+1);
            String tmpText = DocxProcessUtil.getPText(pHeader);
            String nextTmpText = DocxProcessUtil.getPText(pNextHeader);
            if (tmpText.startsWith(ImportPaperMsg.questionClassify_word)) {
            	if(!nextTmpText.startsWith(ImportPaperMsg.questionType_word)){
                	importPaperCheck.setErrorInfo("[试题分类]的下一题头应该是[题型],当前读到的信息为:“" +nextTmpText+"”,请检查word中格式问题");
                	throw new PaperException(importPaperCheck.errorInfo);
                }
                importPaperCheck.setQuesGroup(getContent(tmpText,ImportPaperMsg.questionClassify_word));
            } else if (tmpText.startsWith(ImportPaperMsg.questionType_word)) {
            	if(!nextTmpText.startsWith(ImportPaperMsg.questionName_word)){
                	importPaperCheck.setErrorInfo("[题型]的下一题头应该是[大题名称],当前读到的信息为:“"+nextTmpText+"”,请检查word中格式问题");
                	throw new PaperException(importPaperCheck.errorInfo);
                }
                importPaperCheck.setQuesType(getContent(tmpText,ImportPaperMsg.questionType_word));
            } else if (tmpText.startsWith(ImportPaperMsg.questionName_word)) {
            	if(!nextTmpText.startsWith(ImportPaperMsg.questionNum_word)){
                	importPaperCheck.setErrorInfo("[大题名称]的下一题头应该是[题目数量],当前读到的信息为:“"+nextTmpText+"”,请检查word中格式问题");
                	throw new PaperException(importPaperCheck.errorInfo);
                }
                importPaperCheck.setQuesName(getContent(tmpText,ImportPaperMsg.questionName_word));
            } else if (tmpText.startsWith(ImportPaperMsg.questionNum_word)) {
            	if(!nextTmpText.startsWith(ImportPaperMsg.score_word)){
                	importPaperCheck.setErrorInfo("[题目数量]的下一题头应该是[分数],当前读到的信息为:“"+nextTmpText+"”,请检查word中格式问题");
                	throw new PaperException(importPaperCheck.errorInfo);
                }
                importPaperCheck.setQuesCount(getContent(tmpText,ImportPaperMsg.questionNum_word));
            } else if (tmpText.startsWith(ImportPaperMsg.score_word)) {
                importPaperCheck.setQuesScore(getContent(tmpText,ImportPaperMsg.score_word));
            } else if (StringUtils.isEmpty(tmpText)) {
                continue;
            } else {
                break;
            }
        }
        importPaperCheck.setIndex(i);
    }

    /**
     * 处理题干信息
     *
     * @param pList
     * @param index
     * @param question
     * @param importPaperCheck
     * @param wordMLPackage
     * @throws Exception
     */
    public void processQuesBody(List<Object> pList, int index, int subQuesNum,Question question, ImportPaperCheck importPaperCheck,
            WordprocessingMLPackage wordMLPackage) throws Exception {
        // 定义题干wordml和html
        StringBuilder quesBodyWordMl = new StringBuilder();
        StringBuilder quesBodyHtml = new StringBuilder();
        int i = 0;
        for (i = index; i < pList.size(); i++) {
            P pBody = (P) pList.get(i);
            String tmpText = DocxProcessUtil.getPText(pBody);
            if (StringUtils.isEmpty(tmpText) && DocxProcessUtil.isText(pBody)) {
                // 跳过空白段落
                continue;
            } else if (tmpText.matches("^\\d{1,}\\.[\\s\\S]*")) {
                // 题干第一段
                // 过滤题干标题
                pBody = DocxProcessUtil.formatP(pBody, QuesUnit.QUES_BODY);
                String tmpWordMl = DocxProcessUtil.getPWordMl(pBody);
                tmpWordMl = DocxProcessUtil.formatPWordMl(tmpWordMl);
                quesBodyWordMl.append(tmpWordMl);
                quesBodyHtml.append(DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
            } else if (tmpText.matches("^[a-zA-Z]\\.[\\s\\S]*") || tmpText.startsWith(ImportPaperMsg.left_bracket)) {
                // 检测到选项或其他特殊段落直接退出
                break;
            } else{
                // 题干普通段落
                String tmpWordMl = DocxProcessUtil.getPWordMl(pBody);
                tmpWordMl = DocxProcessUtil.formatPWordMl(tmpWordMl);
                quesBodyWordMl.append(tmpWordMl);
                quesBodyHtml.append(DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
            }
        }
        if(StringUtils.isEmpty(quesBodyHtml) || StringUtils.isEmpty(quesBodyWordMl)){
            if(subQuesNum == 0){
                importPaperCheck.setErrorInfo(importPaperCheck.quesName+ImportPaperMsg.errMsg_01);
            }else{
                importPaperCheck.setErrorInfo(getQuesNumInfo(importPaperCheck.quesName,subQuesNum)+ImportPaperMsg.errMsg_01);
            }
            throw new PaperException(importPaperCheck.errorInfo);
        }
        importPaperCheck.setIndex(i);
        question.setQuesBody(quesBodyHtml.toString());
        question.setQuesBodyWord(quesBodyWordMl.toString());
    }

    /**
     * 处理题目选项
     *
     * @param pList
     * @param index
     * @param subQuesNum
     * @param question
     * @param importPaperCheck
     * @param wordMLPackage
     * @return
     * @throws Exception
     */
    public String processQuesOption(List<Object> pList, int index, int subQuesNum, Question question,
            ImportPaperCheck importPaperCheck, WordprocessingMLPackage wordMLPackage) throws Exception {

        // 定义试题选项wordml和html
        // String quesOptionWordMl = "";
        // String quesOptionHtml = "";
        // 定义选项集合
        List<QuesOption> quesOptions = new ArrayList<QuesOption>();
        // 定义选项数量
        int optionCount = 0;
        // 当前所处的选项
        QuesOption current = null;
        int i = 0;
        int number = 0;
        for (i = index; i < pList.size(); i++) {
            number++;
            P pOption = (P) pList.get(i);
            String tmpText = DocxProcessUtil.getPText(pOption);
            if (StringUtils.isEmpty(tmpText) && DocxProcessUtil.isText(pOption)) {
                // 跳过空白段落
                continue;
            } else if (tmpText.matches("^[a-zA-Z]\\.[\\s\\S]*")) {
                // 检测到选项开始段落
                // 创建试题选项
                current = new QuesOption();
                String optionNumber = String.valueOf(number);
                current.setNumber(optionNumber);
                current.setOptionBody("");
                current.setOptionBodyWord("");
                quesOptions.add(current);
                optionCount++;
                // 过滤选项标题
                pOption = DocxProcessUtil.formatP(pOption, QuesUnit.QUES_OPTION);
                String tmpWordMl = DocxProcessUtil.getPWordMl(pOption);
                current.setOptionBody(current.getOptionBody() + DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
                current.setOptionBodyWord(current.getOptionBodyWord() + DocxProcessUtil.formatPWordMl(tmpWordMl));
            } else if (tmpText.startsWith(ImportPaperMsg.left_bracket)) {
                // 非选项的其他内容直接退出
                break;
            } else if (current != null) {
                // 选项的其他段落
                String tmpWordMl = DocxProcessUtil.getPWordMl(pOption);
                current.setOptionBody(current.getOptionBody() + DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
                current.setOptionBodyWord(current.getOptionBodyWord() + DocxProcessUtil.formatPWordMl(tmpWordMl));
            }else{
                break;
            }
        }
        importPaperCheck.setIndex(i);
        if (optionCount < 2) {
            importPaperCheck.setErrorInfo(getQuesNumInfo(importPaperCheck.quesName, subQuesNum)+ "中选项格式不正确或有缺失\n");
            throw new PaperException(importPaperCheck.errorInfo);
        } else {
            question.setQuesOptions(quesOptions);
        }
        return importPaperCheck.errorInfo;
    }

    /**
     * 处理小题尾信息
     *
     * @param pList
     * @param index
     * @param subQuesNum
     * @param question
     * @param paperDetailUnit
     * @param importPaperCheck
     */
    public void processQuesTail(List<Object> pList, int index, int subQuesNum, Question question,
            PaperDetailUnit paperDetailUnit, ImportPaperCheck importPaperCheck, WordprocessingMLPackage wordMLPackage,
            boolean isNested) throws Exception {

        StringBuilder answerWordML = new StringBuilder("");
        StringBuilder answerHTML = new StringBuilder("");
        String quesScore = "";
        // Map<String, String> quesParams = new HashMap<String, String>();
        int i = 0;
        // 是否刚刚检测到答案内容
        boolean answerStart = false;

        for (i = index; i < pList.size(); i++) {
            P pAnswer = (P) pList.get(i);
            String tmpText = DocxProcessUtil.getPText(pAnswer);
            if (StringUtils.isEmpty(tmpText) && DocxProcessUtil.isText(pAnswer)) {
                // 跳过空白段落
                continue;
            } else if (tmpText.startsWith(ImportPaperMsg.answer_word)) {
                // 检测到答案开始段落
                pAnswer = DocxProcessUtil.formatP(pAnswer, QuesUnit.QUES_ANSWER);
                String tmpWordMl = DocxProcessUtil.getPWordMl(pAnswer);
                if (question.getQuestionType() == QuesStructType.SINGLE_ANSWER_QUESTION
                        || question.getQuestionType() == QuesStructType.MULTIPLE_ANSWER_QUESTION) {
                	//校验单选多选答案
                	checkAnswer(question,pList.get(i).toString(),importPaperCheck,subQuesNum);
                    answerWordML.append(DocxProcessUtil.formatPWordMl(tmpWordMl));
                    answerHTML.append(getContent(tmpText, ImportPaperMsg.answer_word));
                }else if(question.getQuestionType() == QuesStructType.BOOL_ANSWER_QUESTION){
                	answerWordML.append(DocxProcessUtil.formatPWordMl(tmpWordMl));
                    answerHTML.append(getContent(tmpText, ImportPaperMsg.answer_word));
                }else{
                    answerWordML.append(DocxProcessUtil.formatPWordMl(tmpWordMl));
                    answerHTML.append(DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
                }
                answerStart = true;
            } else if (tmpText.startsWith(ImportPaperMsg.subQuesScore_word)) {
                // 检测到分数开始段落
                quesScore = getContent(tmpText, ImportPaperMsg.subQuesScore_word);
                answerStart = false;
            } else if (tmpText.startsWith(ImportPaperMsg.left_bracket) || tmpText.matches("^\\d{1,}\\.[\\s\\S]*")) {
                // 检测到其他特殊段落或下一题直接退出
                break;
            } else if (answerStart) {
                String tmpWordMl = DocxProcessUtil.getPWordMl(pAnswer);
                answerWordML.append(DocxProcessUtil.formatPWordMl(tmpWordMl));
                answerHTML.append(DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
            } else {
                break;
            }
        }
        importPaperCheck.setIndex(i);
        // 设置答案
        if (StringUtils.isNotEmpty(answerHTML)) {
            question.setQuesAnswer(answerHTML.toString());
            question.setQuesAnswerWord(answerWordML.toString());
        }

        // 设置预设分数
        if (quesScore.matches("^\\d+(\\.\\d+)?$")) {
            question.setScore(Double.parseDouble(quesScore));
        }
        // 一般大题明细需要设置分数
        if (!isNested && quesScore.matches("^\\d+(\\.\\d+)?$")) {
            paperDetailUnit.setScore(Double.parseDouble(quesScore));
        }
    }

    /**
     * 处理选择题option信息
     * 
     * @param question
     */
    public void processSelectOption(Question question) {
        String answer = question.getQuesAnswer();
        if (StringUtils.isNotBlank(answer)) {
            String[] answerArray = answer.split(",");
            for (int i = 0; i < question.getQuesOptions().size(); i++) {
                QuesOption quesOption = question.getQuesOptions().get(i);
                char number = (char) (Integer.parseInt(quesOption.getNumber()) + 64);
                if (ArrayUtils.contains(answerArray, String.valueOf(number))) {
                    quesOption.setIsCorrect((short) 1);
                } else {
                    quesOption.setIsCorrect((short) 0);
                }
            }
        }
    }

    /**
     * 处理套题
     *
     * @param pList
     * @param index
     * @param question
     * @param paperDetailUnit
     * @param importPaperCheck
     * @throws Exception
     */
    public void processNestedQues(List<Object> pList, int index,Question question, PaperDetailUnit paperDetailUnit,
            ImportPaperCheck importPaperCheck, WordprocessingMLPackage wordMLPackage) throws Exception {
        // 题型
        String nestedQuesType = "";
        // 设置套题题干
        processQuesBody(pList, index, 0,question, importPaperCheck, wordMLPackage);
        // 创建小题集合
        List<Question> subQuesList = new ArrayList<Question>();
        Question subQues = null;
        int beginNum = importPaperCheck.index;
        int quesTypeNum = 0;
        int subQuesNum = 0;
        for (int i = beginNum; i < pList.size(); i++) {
            P pSubQues = (P) pList.get(i);
            String tmpText = DocxProcessUtil.getPText(pSubQues);
            if (StringUtils.isEmpty(tmpText)) {
                continue;
            }
            if (tmpText.startsWith(ImportPaperMsg.subQuestionType_word)) {
                nestedQuesType = getContent(tmpText, ImportPaperMsg.subQuestionType_word);
                importPaperCheck.index++;
                quesTypeNum++;
            } else if (tmpText.matches("^\\d{1,}\\.[\\s\\S]*")) {
                ++subQuesNum;
                if (StringUtils.isEmpty(nestedQuesType)) {
                    importPaperCheck.setErrorInfo(getQuesNumInfo(importPaperCheck.quesName, subQuesNum) + ImportPaperMsg.errMsg_02);
                    throw new PaperException(importPaperCheck.errorInfo);
                }
                subQues = new Question();
                subQues.setQuestionType(getQuesStructType(nestedQuesType));
                if (StringUtils.isNumeric(importPaperCheck.getQuesScore())) {
                    subQues.setScore(Double.parseDouble(importPaperCheck.getQuesScore()));
                }
                // 处理客观题
                if (nestedQuesType.equals(ImportPaperMsg.singleSelection) || nestedQuesType.equals(ImportPaperMsg.multipleSelection)) {
                    // 处理题干
                    processQuesBody(pList, importPaperCheck.index, subQuesNum,subQues, importPaperCheck, wordMLPackage);
                    // 处理选项
                    processQuesOption(pList, importPaperCheck.index, subQuesNum, subQues, importPaperCheck,wordMLPackage);
                    // 处理尾信息
                    processQuesTail(pList, importPaperCheck.index, subQuesNum, subQues, paperDetailUnit,importPaperCheck, wordMLPackage, true);
                    // 处理选择题的option--chenken 20170426
                    processSelectOption(subQues);
                } else {
                    // 处理其他题型
                    processQuesBody(pList, importPaperCheck.index, subQuesNum, subQues, importPaperCheck, wordMLPackage);
                    processQuesTail(pList, importPaperCheck.index, subQuesNum, subQues, paperDetailUnit,importPaperCheck, wordMLPackage, true);
                }
                subQuesList.add(subQues);
                i = importPaperCheck.index - 1;
                nestedQuesType = "";
            }else if(subQuesNum != quesTypeNum){
                if (StringUtils.isEmpty(nestedQuesType)) {
                    importPaperCheck.setErrorInfo(getQuesNumInfo(importPaperCheck.quesName, quesTypeNum) + ImportPaperMsg.errMsg_02);
                }else{
                    importPaperCheck.setErrorInfo(getQuesNumInfo(importPaperCheck.quesName, quesTypeNum) + ImportPaperMsg.errMsg_03);
                }
                throw new PaperException(importPaperCheck.errorInfo);
            }else if(tmpText.startsWith(ImportPaperMsg.left_bracket)){
                break;
            }
            if (!StringUtils.isEmpty(importPaperCheck.errorInfo)) {
                return;
            }
        }
        question.setSubQuestions(subQuesList);
        // 计算套题总分
        double totalScore = 0;
        List<Double> scoreList = new ArrayList<>();
        for (Question child : subQuesList) {
            Double score = child.getScore();
            if (score != null) {
                totalScore += score;
                scoreList.add(score);
            } else {
                scoreList.add(0d);
            }
        }
        question.setScore(totalScore);
        paperDetailUnit.setSubScoreList(scoreList);
    }

    /**
     * 获取试题类型
     *
     * @param quesType
     * @return
     */
    public static QuestionType getQuesType(String quesType) {
        return getQuesType(quesType, "");
    }

    /**
     * 获取试题类型
     *
     * @param quesType
     * @param quesName
     * @return
     */
    public static QuestionType getQuesType(String quesType, String quesName) {
        QuesStructType quesStructType = CommonUtils.getEnum(QuesStructType.class, quesType);
        return new QuestionType(quesStructType, quesName);
    }

    /**
     * 获取试题结构类型
     *
     * @param quesType
     * @return
     */
    public static QuesStructType getQuesStructType(String quesType) {
        return CommonUtils.getEnum(QuesStructType.class, quesType);
    }

    /**
     * 获取当前试题大题小题号
     *
     * @param quesType
     * @param subQuesNum
     * @return
     */
    public static String getQuesNumInfo(String quesType, int subQuesNum) {
        return quesType + "第" + subQuesNum + "小题";
    }

    /**
     * 判断是否为套题
     *
     * @param importPaperCheck
     * @return
     */
    public static boolean isNested(ImportPaperCheck importPaperCheck) {
        if (importPaperCheck.getQuesType().equals(ImportPaperMsg.nestedQuestion_word)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 获取课程代码
     *
     * @param importPaperCheck
     * @return
     */
    public static String getCourseNo(ImportPaperCheck importPaperCheck) {
        String[] courseInfo = importPaperCheck.quesGroup.split("_");
        if (courseInfo.length == 2) {
            return courseInfo[1];
        } else {
            return "";
        }
    }

    /**
     * 获取课程名称
     *
     * @param importPaperCheck
     * @return
     */
    public static String getCourseName(ImportPaperCheck importPaperCheck) {
        String[] courseInfo = importPaperCheck.quesGroup.split("_");
        if (courseInfo.length == 2) {
            return courseInfo[0];
        } else {
            return "";
        }
    }

    /**
     * 设置wordMLPackage对象二进制数据
     *
     * @param question
     * @param wordMLPackage
     * @throws Exception
     */
    public static void setPkgByte(Question question, WordprocessingMLPackage wordMLPackage) throws Exception {
        question.setQuesPkg(DocxProcessUtil.getPkgByte(wordMLPackage));
    }

    /**
     * 构造一张空白的导入类型试卷
     *
     * @param courseNo
     * @param paperName
     * @return
     */
    public Map<String, Object> saveBlankPaper(String courseNo, String courseName, String paperName, AccessUser user) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
        Paper paper = new Paper();
        String msg = paperService.checkPaperName(paperName, user.getRootOrgId().toString());
        if (msg != null) {
            returnMap.put("msg", msg);
            return returnMap;
        }
        initPaper(paper, paperName);
        paper.setCourseNo(courseNo);
        paper.setCourseName(courseName);
        paper.setOrgId(user.getRootOrgId().toString());
        paper.setCreator(user.getName());
        paper.setCreateTime(CommonUtils.getCurDateTime());
        PaperDetail pd = new PaperDetail();
        pd.setId(UUID.randomUUID().toString());
        pd.setNumber(1);
        pd.setPaper(paper);
        pd.setCreator(user.getName());
        pd.setCreateTime(CommonUtils.getCurDateTime());
        paper = paperRepo.save(paper);
        paperDetailRepo.save(pd);
        returnMap.put("paper", paper);
        returnMap.put("msg", "success");
        return returnMap;
    }
    /**
     * 校验试卷总分是否与设定一致
     * @param paper
     * @param paperDetailUnits
     * @param importPaperCheck
     * @throws PaperException
     */
    private void checkTotalScore(Paper paper,List<PaperDetailUnit> paperDetailUnits,ImportPaperCheck importPaperCheck) throws PaperException{
    	double totalScore = 0;
        for (PaperDetailUnit unit : paperDetailUnits) {
            if (unit.getScore() != null) {
                totalScore += unit.getScore();
            }
        }
    	if(paper.getTotalScore()!=null&&paper.getTotalScore()!=totalScore){
    		importPaperCheck.setErrorInfo("试卷总分("+totalScore+")与设置总分("+paper.getTotalScore()+")不符");
    		throw new PaperException(importPaperCheck.errorInfo);
    	}
    }
    
    /**
     * 校验大题下小题数量是否与标识数量一致
     * @throws PaperException 
     */
    private void checkUnitNum(List<PaperDetailUnit> paperDetailUnits,ImportPaperCheck importPaperCheck) throws PaperException{
    	Map<PaperDetail, Integer> unitNumMap = new HashMap<PaperDetail, Integer>();
    	Collections.sort(paperDetailUnits);
    	for(PaperDetailUnit paperDetailUnit:paperDetailUnits){
    		PaperDetail key = paperDetailUnit.getPaperDetail();
    		if(unitNumMap.containsKey(key)){
    			int value = unitNumMap.get(key);
    			if(paperDetailUnit.getQuestionType()==QuesStructType.NESTED_ANSWER_QUESTION){
    				unitNumMap.put(key,value+paperDetailUnit.getSubScoreList().size());
    			}else{
    				unitNumMap.put(key, value+1);
    			}
    		}else{
    			if(paperDetailUnit.getQuestionType()==QuesStructType.NESTED_ANSWER_QUESTION){
    				unitNumMap.put(key,paperDetailUnit.getSubScoreList().size());
    			}else{
    				unitNumMap.put(key, 1);
    			}
    		}
    	}
    	for (Map.Entry<PaperDetail, Integer> entry : unitNumMap.entrySet()) {
    		PaperDetail paperDetail = entry.getKey();
    		if(paperDetail.getUnitCount().intValue()!=entry.getValue().intValue()){
    			importPaperCheck.setErrorInfo("大题:"+paperDetail.getName()
    												+"标识小题数量("+paperDetail.getUnitCount()
    												+")与实际小题数量("+entry.getValue()+")不符。"
    												+ "(注:若检查小题数量无问题,请检查word中题目的格式是否正确)");
        		throw new PaperException(importPaperCheck.errorInfo);
    		}
    	}
    }
    /**
     * 校验选择题答案
     * 1.判断答案格式，选择题答案必须以","分隔,且都为字母
     * 2.判断答案在选项中是否不存在,例如选项为A,B,C,D，答案为E
     * @param question
     * @param answerWord
     * @param importPaperCheck
     * @throws PaperException 
     */
    private void checkAnswer(Question question,String answerWord,ImportPaperCheck importPaperCheck,int subQuesNum) throws PaperException{
    	if(!StringUtils.isBlank(answerWord)){
    		String[] pAnswerArray = answerWord.split(",");
    		List<QuesOption> options = question.getQuesOptions();
        	List<String> optionNumList = new ArrayList<String>();
        	for(QuesOption quesOption:options){
        		Integer numInteger = Integer.parseInt(quesOption.getNumber());
        		char word = (char)(numInteger+64);
        		optionNumList.add(word+"");
    		}
        	for(String answer:pAnswerArray){
        		answer = answer.trim();
        		String pattern = "[A-Z]|[a-z]";
        		if(!Pattern.matches(pattern,answer)){
        			importPaperCheck.setErrorInfo(getQuesNumInfo(importPaperCheck.quesName, subQuesNum)+
       					 "中,答案格式不正确,答案为:"+answerWord);
       	         	throw new PaperException(importPaperCheck.errorInfo);
        		}
        		if(!optionNumList.contains(answer)){
        			 importPaperCheck.setErrorInfo(getQuesNumInfo(importPaperCheck.quesName, subQuesNum)+
        					 "中,选项为:"+optionNumList.toString()+",答案为:"+answerWord+
        					 "。(注:若检查选项无问题,请检查word中题目的格式是否正确)");
        	         throw new PaperException(importPaperCheck.errorInfo);
        		}
        	}
    	}
    }
    /**
     * 校验客观题答案完整性
     * 客观题答案要么全有，要么全没有
     * @throws PaperException 
     */
    private void checkAnswerISfull(List<PaperDetailUnit> paperDetailUnits,ImportPaperCheck importPaperCheck) throws PaperException{
    	List<PaperDetailUnit> paperDetailUnitList = new ArrayList<PaperDetailUnit>();
    	for(PaperDetailUnit paperDetailUnit:paperDetailUnits){
    		Question question = paperDetailUnit.getQuestion();
    		if(question.getQuestionType() == QuesStructType.SINGLE_ANSWER_QUESTION
                        || question.getQuestionType() == QuesStructType.MULTIPLE_ANSWER_QUESTION
                        || question.getQuestionType() == QuesStructType.BOOL_ANSWER_QUESTION){
    			paperDetailUnitList.add(paperDetailUnit);
    		}
    	}
    	boolean isNull = false;
    	boolean isNotNull = false;
    	PaperDetailUnit paperDetailUnit = new PaperDetailUnit();
    	for(PaperDetailUnit pdu:paperDetailUnitList){
    		Question question = pdu.getQuestion();
    		if(StringUtils.isBlank(question.getQuesAnswer())){
    			isNull = true;
    			paperDetailUnit = pdu;
    			break;
    		}
    	}
    	for(PaperDetailUnit pdu:paperDetailUnitList){
    		Question question = pdu.getQuestion();
    		if(StringUtils.isNotBlank(question.getQuesAnswer())){
    			isNotNull = true;
    			break;
    		}
    	}
    	//不满足(客观题答案要么全有，要么全没有)条件
    	if(isNull&&isNotNull){
    		importPaperCheck.setErrorInfo("第"+paperDetailUnit.getNumber()+"小题答案缺失,请检查");
	        throw new PaperException(importPaperCheck.errorInfo);
    	}
    }
    
    /**
     * 替换字符串中的属性头得到属性值
     * @param pText
     * @param replaceContent
     * @return
     */
    private static String getContent(String pText,String replaceContent){
    	String word = replaceContent.replace("[", "").replace("]", "");
    	return pText.replaceAll("\\["+word+"\\]","").replaceAll("[:|：]", "").trim();
    }
    public static void main(String[] args){
    	String answer = "a";
    	String pattern = "[A-Z]|[a-z]";
    	System.out.println(Pattern.matches(pattern,answer));
    }
}
