package com.qmth.cqb.paper.service.export;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;

import com.google.gson.Gson;
import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.dto.PaperDetailExp;
import com.qmth.cqb.paper.dto.PaperDetailUnitExp;
import com.qmth.cqb.paper.dto.PaperExp;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.paper.service.PaperDetailUnitService;
import com.qmth.cqb.paper.service.PaperService;
import com.qmth.cqb.question.dao.QuesRepo;
import com.qmth.cqb.question.model.QuesOption;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.utils.BeanCopierUtil;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.exception.PaperException;
import com.qmth.cqb.utils.word.DocxProcessUtil;

import org.apache.commons.lang3.StringUtils;
import org.docx4j.XmlUtils;
import org.docx4j.jaxb.Context;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.wml.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Created by songyue on 17/3/15.
 */
@Service
public class ExportPaperService extends ExportPaperAbstractService{

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    PaperDetailRepo paperDetailRepo;

    @Autowired
    PaperDetailUnitRepo paperDetailUnitRepo;

    @Autowired
    QuesRepo quesRepo;

    @Autowired
    Gson gson;

    @Autowired
    PaperDetailUnitService paperDetailUnitService;

    @Autowired
    PaperService paperService;
    
    /**
     * 初始化导出试卷DTO
     * @param id
     * @return
     */
    public Map initExportPaper(String id) throws Exception{
        //创建返回Map
        Map returnMap = new HashMap();
        //获取paper
        Paper paper = paperRepo.findOne(id);

        paperService.formatPaper(paper,null);

        if(paper == null){
            throw new PaperException("该试卷不存在");
        }

        //创建paperDto
        PaperExp paperExp = BeanCopierUtil.copyProperties(paper,PaperExp.class);

        paperExp.setTitle(CommonUtils.PAPER_TITLE);
        paperExp.setSubTitle(CommonUtils.PAPER_SUB_TITLE);
        //获取大题
        List<PaperDetail> paperDetails = paperDetailRepo.findByPaper(paper);
        List<PaperDetailExp> paperDetailExps =
                BeanCopierUtil.copyPropertiesOfList(paperDetails,PaperDetailExp.class);
        paperExp.setPaperDetails(paperDetailExps);

        //封装小题
        for(int i = 0; i < paperDetailExps.size(); i++){
            PaperDetailExp paperDetailExp = paperDetailExps.get(i);
            paperDetailExp.setTitle(getDetailTitle(paperDetailExp));
            List<PaperDetailUnit> paperDetailUnits =
                    paperDetailUnitRepo.findByPaperDetail(paperDetails.get(i));

            setAnswerWord(paperDetailUnits);

            List<PaperDetailUnitExp> paperDetailUnitExps =
                    BeanCopierUtil.copyPropertiesOfList(paperDetailUnits,PaperDetailUnitExp.class);
            //选择题，套题下选择题  选项顺序重新排列
            paperService.reorderChoicequestionOption(paperDetailUnitExps);
            paperDetailExps.get(i).setPaperDetailUnits(paperDetailUnitExps);
        }
        //初始化试卷序号
        initPaperNum(paperExp);

        returnMap.put("paper", paperExp);
        returnMap.put("fileName", paperExp.getName());

        return returnMap;
    }

    /**
     * 设置答案
     * @param answer
     * @return
     * @throws Exception
     */
    public String getObjectAnswer(String answerWordMl,String answer) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + answerWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,
                Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for(Object pObj:pList){
            if(index > 0){
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            pContent.removeAll(pContent);
            R run = new R();
            Text text = new Text();
            text.setValue("[答案]:"+answer);
            run.getContent().add(text);
            pContent.add(0,run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }

    /**
     * 设置答案
     * @param answerWordMl
     * @return
     * @throws Exception
     */
    public String getSubjectAnswer(String answerWordMl) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + answerWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,
                Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for(Object pObj:pList){
            if(index > 0){
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            R run = new R();
            Text text = new Text();
            text.setValue("[答案]:");
            run.getContent().add(text);
            pContent.add(0,run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }

    private void setAnswerWord(List<PaperDetailUnit> paperDetailUnits) throws Exception{
        for (PaperDetailUnit paperDetailUnit : paperDetailUnits) {
            if(paperDetailUnit==null||paperDetailUnit.getQuestion()==null){
                continue;
            }
            String optionOrder = paperDetailUnit.getOptionOrder();
            Question question = paperDetailUnit.getQuestion();
            if (question.getQuestionType() == QuesStructType.SINGLE_ANSWER_QUESTION
                    || question.getQuestionType() == QuesStructType.MULTIPLE_ANSWER_QUESTION) {
                setAnswerWordUnit(question, optionOrder);
            } else if (question.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                List<Question> subQuestions = question.getSubQuestions();
                int index = 0;
                for (int k = 0; k < subQuestions.size(); k++) {
                    Question subQuestion = subQuestions.get(k);
                    if (subQuestion.getQuestionType() == QuesStructType.SINGLE_ANSWER_QUESTION
                            || subQuestion.getQuestionType() == QuesStructType.MULTIPLE_ANSWER_QUESTION) {
                        if(StringUtils.isNotEmpty(optionOrder) && optionOrder.contains(";")){
                            setAnswerWordUnit(subQuestion, optionOrder.split(";")[index]);
                        }else{
                            setAnswerWordUnit(subQuestion, "");
                        }
                        index++;
                    }else{
                        subQuestion.setQuesAnswerWord(getSubjectAnswer(subQuestion.getQuesAnswerWord()));
                    }
                }
            }else{
                question.setQuesAnswerWord(getSubjectAnswer(question.getQuesAnswerWord()));
            }
        }
    }

    private void setAnswerWordUnit(Question question, String optionOrder) throws Exception{
        List<QuesOption> quesOptions = question.getQuesOptions();
        if(quesOptions == null || quesOptions.size() == 0){
            return;
        }
        if(StringUtils.isEmpty(optionOrder)){
            int j = 0;
            String answer = "";
            for(QuesOption quesOption : quesOptions){
                if(quesOption.getIsCorrect() == 1){
                    answer += CommonUtils.getOptionNum(j);
                }
                j++;
            }
            question.setQuesAnswerWord(getObjectAnswer(question.getQuesAnswerWord(),answer));
        }else{
            String [] order = optionOrder.split(",");
            for(int i = 0;i < order.length;i++){
                for(QuesOption quesOption : quesOptions){
                    if(order[i].equals(quesOption.getNumber()) && quesOption.getIsCorrect() == 1){
                        question.setQuesAnswerWord(getObjectAnswer(question.getQuesAnswerWord(),CommonUtils.getOptionNum(i)));
                    }
                }
            }
        }
    }

    public String getDetailTitle(PaperDetailExp paperDetailExp){
        int totalScore = paperDetailExp.getScore().intValue();
        int count = paperDetailExp.getUnitCount();
        String title = "本大题共"+count+"小题,共"+totalScore+"分";
        return title;
    }

    /**
     * 导出试卷
     * @param id
     */
    public void exportPaper(String id) throws Exception {
//        Map dataMap = initExportPaper(id);
//        if(dataMap.get("fileName") != null){
//            String fileName = (String)dataMap.get("fileName");
//            DocxProcessUtil.exportPaper(dataMap,fileName);
//            DocxProcessUtil.processImage(fileName,getPkgList(id));
//        }
    }

    /**
     * 下载试卷
     * @param id
     */
    public void downloadPaper(String id, HttpServletResponse response) throws Exception {
    	List<String> fileNames = new ArrayList<String>();
         Map dataMap = initExportPaper(id);
        
        if (dataMap.get("fileName") != null) {
            String fileName = (String) dataMap.get("fileName");
            DocxProcessUtil.exportPaper(dataMap, fileName,TEMPLATE_NORMAL);
            DocxProcessUtil.processImage(fileName, getPkgList(id));
            DocxProcessUtil.processDownload(fileNames, response);
        }
    }

    /**
     * 初始化序号
     * @param paperExp
     */
    public void initPaperNum(PaperExp paperExp)throws Exception{
        if(paperExp.getPaperDetails() == null || paperExp.getPaperDetails().size() == 0){
            return;
        }
        int mainNum = 0;
        int subNum = 0;
        Collections.sort(paperExp.getPaperDetails());
        for(PaperDetailExp paperDetail: paperExp.getPaperDetails()){
            //大题序号
            paperDetail.setNumber(++mainNum);
            paperDetail.setCnNum(CommonUtils.toCHNum(paperDetail.getNumber()));
            if(paperDetail.getPaperDetailUnits() == null ||
                    paperDetail.getPaperDetailUnits().size() == 0){
                continue;
            }
            //小题序号
            for(PaperDetailUnitExp paperDetailUnit:paperDetail.getPaperDetailUnits()){
                List<QuesOption> optionList = paperDetailUnit.getQuestion().getQuesOptions();
                if(optionList != null && optionList.size() > 0){
                    int index = 0;
                    for(QuesOption quesOption:optionList){
                        quesOption.setOptionBodyWord(setOptionNum(quesOption.getOptionBodyWord(),getOptionNum(index)));
                        index++;
                    }
                }
                List<Question> subQuesList = paperDetailUnit.getQuestion().getSubQuestions();
                Question question = paperDetailUnit.getQuestion();
                //套题序号
                if(subQuesList != null && subQuesList.size() > 0){

                    question.setQuesBodyWord(replaceQuesBlank(question.getQuesBodyWord(),subNum + 1));

                    for(Question subQues:subQuesList){
                        int curSubNum = ++subNum;
                        subQues.setQuesBodyWord(setSubQuesNum(subQues.getQuesBodyWord(),curSubNum));
                        subQues.setQuesBodyWord(replaceQuesBlank(subQues.getQuesBodyWord(),curSubNum));
                        List<QuesOption> subOptionList = subQues.getQuesOptions();
                        if(subOptionList != null && subOptionList.size() > 0){
                            int sub_index = 0;
                            for(QuesOption quesOption:subOptionList){
                                quesOption.setOptionBodyWord(setOptionNum(quesOption.getOptionBodyWord(),getOptionNum(sub_index)));
                                sub_index++;
                            }
                        }
                    }
                }else{
                    int curSubNum = ++subNum;
                    question.setQuesBodyWord(setSubQuesNum(question.getQuesBodyWord(),curSubNum));
                    question.setQuesBodyWord(replaceQuesBlank(question.getQuesBodyWord(),curSubNum));
                }
            }
        }
    }

    public String getOptionNum(int number){
        char optionNum = (char)(65 + number);
        return String.valueOf(optionNum);
    }

    /**
     * 获取当前试卷下所有试题WordPkg
     * @param id
     * @return
     */
    public List<WordprocessingMLPackage> getPkgList(String id){
        Paper paper = paperRepo.findOne(id);
        List<WordprocessingMLPackage> wordMLPackages = paperDetailUnitRepo.findByPaper(paper)
                .stream().map(PaperDetailUnit::getQuestion).collect(Collectors.toList())
                .stream().map(Question::getPkgObj).collect(Collectors.toList());
        return wordMLPackages;
    }

    /**
     * 设置题号
     * @param quesBodyWordMl
     * @param num
     * @return
     * @throws Exception
     */
    public String setSubQuesNum(String quesBodyWordMl,int num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + quesBodyWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,
                Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for(Object pObj:pList){
            if(index > 0){
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            R run = new R();
            Text text = new Text();
            text.setValue(num+". ");
            run.getContent().add(text);
            pContent.add(0,run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }

    /**
     * 设置选项号
     * @param optionWordMl
     * @param num
     * @return
     * @throws Exception
     */
    public String setOptionNum(String optionWordMl,String num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + optionWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,
                Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for(Object pObj:pList){
            if(index > 0){
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            R run = new R();
            Text text = new Text();
            text.setValue(num+". ");
            run.getContent().add(text);
            pContent.add(0,run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }

    /**
     * 替换填空
     * @param wordMl
     * @param num
     * @return
     * @throws Exception
     */
    public String replaceQuesBlank(String wordMl,int num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + wordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,
                Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        int cur = 0;
        Map<Integer,String> curMap = new HashMap<Integer,String>();
        for(Object pObj:pList){
            if(pObj.getClass().equals(P.class)){
                List<Object> pContent = ((P)pObj).getContent();
                for(Object rObj:pContent){
                    if(rObj.getClass().equals(R.class)){
                        List<Object> rContent = ((R)rObj).getContent();
                        for(Object tObj:rContent){
                            if (tObj instanceof JAXBElement)
                                tObj = ((JAXBElement<?>) tObj).getValue();
                            if(tObj.getClass().equals(Text.class)){
                                Text text = (Text)tObj;
                                String str = text.getValue().trim();
                                if(str.equals("###")){
                                    text.setValue("______");
                                }else if(str.matches("##\\d{1,}##")){
                                    int curNum = num + index;
                                    text.setValue("___"+(curNum)+"___");
                                    index++;
                                }else if(str.startsWith("#") || str.equals("___")){
                                    curMap.put(cur,str);
                                    text.setValue("");
                                }else if(str.matches("^\\d{1,}$")){
                                    String preStr = curMap.get(cur - 1);
                                    if(!StringUtils.isEmpty(preStr) && preStr.startsWith("#")){
                                        int curNum = num + index;
                                        text.setValue("___"+(curNum)+"___");
                                        index++;
                                    }
                                }
                                cur++;
                            }
                        }
                    }
                }
            }
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }

}
