package com.qmth.cqb.paper.web;

import io.swagger.annotations.ApiOperation;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections4.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

import cn.com.qmth.examcloud.common.uac.annotation.Uac;
import cn.com.qmth.examcloud.common.uac.enums.RoleMeta;
import cn.com.qmth.examcloud.common.uac.enums.UacPolicy;

import com.qmth.cqb.paper.model.ExamPaper;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.service.ExamPaperService;

/**
 * @author  	chenken
 * @date    	2017年4月17日 上午10:40:20
 * @company 	QMTH
 * @description ExamPaperController.java
 */
@Controller
@RequestMapping("${api_cqb}/")
public class ExamPaperController {
	
	@Autowired
	private ExamPaperService examPaperService;
	
	@ApiOperation(value = "根据考试ID和课程代码获取卷库考试试卷，按试卷类型分组"
				, notes = "根据考试ID和课程代码获取卷库考试试卷，按试卷类型分组")
    @GetMapping(value = "/examPaper/listByGroupCode/{examId}/{courseCode}")
	public ResponseEntity<Map<String, List<Paper>>> findExamPaper(@PathVariable Long examId, @PathVariable String courseCode){
		ExamPaper condition = new ExamPaper();
		condition.setExamId(examId);
		condition.setCourseCode(courseCode);
		//根据考试ID和课程code取出所有试卷
		List<ExamPaper> examPaperList = examPaperService.findExamPaperList(condition);
		//按照试卷类型分组
		Map<String,List<Paper>> examPaperMap = new HashedMap<String,List<Paper>>();
		for(int i = 0;i<examPaperList.size();i++){
			ExamPaper examPaper = examPaperList.get(i);
			if(!examPaperMap.containsKey(examPaper.getGroupCode())){
				if(examPaper.getPaper()!=null){
					List<Paper> epList = new ArrayList<Paper>();
					epList.add(examPaper.getPaper());
					examPaperMap.put(examPaper.getGroupCode(), epList);
				}
			}else{
				if(examPaper.getPaper()!=null){
					List<Paper> epList = examPaperMap.get(examPaper.getGroupCode());
					epList.add(examPaper.getPaper());
				}
			}
		}
		return new ResponseEntity<Map<String, List<Paper>>>(examPaperMap, HttpStatus.OK);
	}
}

