package com.qmth.cqb.paper.web;


import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import cn.com.qmth.examcloud.common.dto.question.PaperDto;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

import com.qmth.cqb.paper.model.ExtractConfig;
import com.qmth.cqb.paper.service.ExtractConfigService;


/**
 * 
 * @author  	chenken
 * @date    	2017年4月14日 下午6:05:37
 * @company 	QMTH
 * @description 调卷规则控制器
 */
@Controller
@RequestMapping("${api_cqb}/")
public class ExtractConfigController {
	private static final Logger logger = LoggerFactory.getLogger(ExtractConfigController.class);
	@Autowired
	private ExtractConfigService extractConfigService;
	
	@ApiOperation(value = "根据考试ID和课程ID获取调卷规则", notes = "根据考试ID和课程ID获取调卷规则")
    @GetMapping(value = "/extractConfig/{examId}/{courseCode}")
	public ResponseEntity<ExtractConfig> findExtractConfig(@PathVariable Long examId,@PathVariable String courseCode){
		ExtractConfig condition = new ExtractConfig();
		condition.setExamId(examId);
		condition.setCourseCode(courseCode);
		ExtractConfig extractConfig = extractConfigService.findConfig(condition);
		return new ResponseEntity<ExtractConfig>(extractConfig,HttpStatus.OK);
	}
	
	@ApiOperation(value = "根据ID获取调卷规则", notes = "根据ID获取调卷规则")
    @GetMapping(value = "/extractConfig/{id}")
	public ResponseEntity<ExtractConfig> findExtractConfigById(@PathVariable String id){
		ExtractConfig extractConfig = extractConfigService.findConfigById(id);
		return new ResponseEntity<ExtractConfig>(extractConfig,HttpStatus.OK);
	} 
	
	@ApiOperation(value = "保存调卷规则", notes = "保存调卷规则")
    @PutMapping(value = "/extractConfig")
	public ResponseEntity saveExtractConfig(HttpServletRequest request,@RequestBody ExtractConfig extractConfig){
		try{
			AccessUser user = (AccessUser) request.getAttribute("accessUser");
			extractConfig.setOrgId(user.getRootOrgId()+"");
			extractConfigService.saveExtractConfig(extractConfig);
			return new ResponseEntity(HttpStatus.OK);
		}catch(Exception e){
			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
	
	@ApiOperation(value = "根据调卷规则生成试卷", notes = "根据调卷规则生成试卷")
    @PutMapping(value = "/makePaperByConfig/{id}")
	public ResponseEntity makePaperByConfig(@PathVariable String id){
		try{
			extractConfigService.makePaperByConfig(id);
			return new ResponseEntity(HttpStatus.OK);
		}catch(Exception e){
			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
	
	@ApiOperation(value = "抽取试卷", notes = "抽取试卷")
    @GetMapping(value = "/extract/{exam_id}/{course_code}/{group_code}")
    public ResponseEntity extract(@PathVariable Long exam_id, @PathVariable String course_code,@PathVariable String group_code) {
		try{
			Map<String, Object> returnMap = extractConfigService.extract(exam_id, course_code, group_code);
			if(returnMap.get("errorMsg")==null){
				PaperDto paperDto = (PaperDto) returnMap.get("paperDto");
				return new ResponseEntity<PaperDto>(paperDto, HttpStatus.OK);
			}else{
				return new ResponseEntity<String>(returnMap.get("errorMsg")+"", HttpStatus.OK);
			}
		}catch(Exception e){
			logger.error("抽卷失败",e);
			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
	
	@ApiOperation(value = "抽取单个试题", notes = "抽取单个试题")
    @GetMapping(value = "/extractQues/{paperDetailUnitId}")
    public ResponseEntity extractQuestion(@PathVariable String paperDetailUnitId) {
        Map<String, Object> quesMap = new HashMap<String, Object>();
        quesMap = extractConfigService.getQuestionById(paperDetailUnitId);
        if (quesMap.get("msg").equals("success")) {
            return new ResponseEntity(quesMap.get("quesDto"), HttpStatus.OK);
        } else {
            return new ResponseEntity(quesMap.get("msg"), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
	
	@ApiOperation(value = "判断试卷中的题是否都为客观题(单选、多选、判断)，包括套题中的小题", 
				  notes = "判断试卷中的题是否都为客观题(单选、多选、判断)，包括套题中的小题")
    @GetMapping(value = "/checkObjective/{paperId}")
	public ResponseEntity<Map<String, Object>> checkIsAllObjectiveQuestion(@PathVariable String paperId){
		Map<String, Object> quesMap = new HashMap<String, Object>();
		try{
			if(StringUtils.isBlank(paperId)){
				quesMap.put("message","paperId不能为空");
				return new ResponseEntity<Map<String, Object>>(HttpStatus.INTERNAL_SERVER_ERROR);
			}
			boolean result = extractConfigService.checkIsAllQbjectiveQuestion(paperId);
			quesMap.put("result", result);
			quesMap.put("message", result?"全为客观题":"不全为客观题");
			return new ResponseEntity<Map<String, Object>>(quesMap, HttpStatus.OK);
		}catch(Exception e){
			logger.error("调用checkIsAllQbjectiveQuestion失败",e);
			quesMap.put("result","error");
			quesMap.put("message","调用失败");
			return new ResponseEntity<Map<String, Object>>(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
}	
